
-- VHDL Test Bench Created from source file rx.vhd -- 12:34:00 11/03/2004
--
--
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY rx_tb IS
END rx_tb;

ARCHITECTURE behavior OF rx_tb IS 

        CONSTANT clk_period         : time      := 20 ns;
        CONSTANT ck_period_mult     : integer   := 3;
        CONSTANT rx_shift           : time      := 30.3 ns; -- creates a nice shift. 
                                                            -- Must be larger than clk_period

        CONSTANT ck_en_period       : time      := clk_period * ck_period_mult;

	COMPONENT rx
	PORT(
		rx_in : IN std_logic;
		clk : IN std_logic;
		ck_en : IN std_logic;
		reset : IN std_logic;
		full_clr : IN std_logic;          
		d_out : OUT std_logic_vector(7 downto 0);
		full : OUT std_logic
		);
	END COMPONENT;

	SIGNAL rx_in :  std_logic;
	SIGNAL clk :  std_logic;
	SIGNAL ck_en :  std_logic;
	SIGNAL reset :  std_logic;
	SIGNAL d_out :  std_logic_vector(7 downto 0);
	SIGNAL full :  std_logic;
	SIGNAL full_clr :  std_logic;
	SIGNAL err :  std_logic := '0';


BEGIN

	uut: rx PORT MAP(
		rx_in => rx_in,
		clk => clk,
		ck_en => ck_en,
		reset => reset,
		d_out => d_out,
		full => full,
		full_clr => full_clr
	);



   clk_p : PROCESS
	BEGIN
	    CLOCK_LOOP : LOOP
		clk <=  '0';
		WAIT FOR clk_period/2;
		clk <= '1';
		WAIT FOR clk_period/2;
            END LOOP CLOCK_LOOP;
	END PROCESS;

   ck_en_p : PROCESS
	BEGIN
            ck_en <= '0';
            WAIT FOR clk_period/4;
	    CK_EN_LOOP : LOOP
		ck_en <=  '1';
		WAIT FOR clk_period;
		ck_en <= '0';
		WAIT FOR (ck_en_period - clk_period);
            END LOOP CK_EN_LOOP;
	END PROCESS;

   reset_p : PROCESS
	BEGIN
            reset <= '1';
            WAIT FOR 45 ns;
            reset <='0';
            WAIT;
	END PROCESS;


   rx_p : PROCESS
	BEGIN
            full_clr <= '0';
            rx_in <= '1';

            err <= '0';

            WAIT FOR 100 ns;

	    RX_LOOP : LOOP

                rx_in <='0';
                WAIT FOR 3*ck_en_period; --START

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B7

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B6

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B5

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B4

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B3

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B2

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B1

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B0

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- STOP

                if (d_out /= "01101001") then
                    err <= '1';
                end if;


                WAIT FOR rx_shift;
                    full_clr <= '0'; -- see what if the full is not cleared
                WAIT FOR rx_shift;
                    full_clr <= '0';
                WAIT FOR rx_shift;

                rx_in <='0';
                WAIT FOR 3*ck_en_period; --START

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B7

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B6

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B5

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B4

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B3

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B2

                rx_in <='0';
                WAIT FOR 3*ck_en_period; -- B1

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- B0

                rx_in <='1';
                WAIT FOR 3*ck_en_period; -- STOP

                if (d_out /= "10001110") then
                    err <= '1';
                end if;

                WAIT FOR rx_shift;
                    full_clr <= '1';
                WAIT FOR rx_shift;
                    full_clr <= '0';
                WAIT FOR rx_shift;

            END LOOP RX_LOOP;
	END PROCESS;


END;
