/*------------------------------------------------------------------------- 
File name   : uart_monitor.e
Title       : uart eVC cell structure
Project     : uart eVC
Developers  : Pinhas Krengel http://bknpk.no-ip.biz/my_web/index.html
Created     : Dec 2011
Description : The monitor unit implements a monitor for rx serail data.
Notes       : 
--------------------------------------------------------------------------- 
==== This source file may be used and distributed without ====
==== restriction provided that this copyright statement is not ====
==== removed from the file and that any derivative work contains ====
==== the original copyright notice and the associated disclaimer.====
==== ====
==== THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY ====
==== EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED ====
==== TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS ====
==== FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR ====
==== OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, ====
==== INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ====
==== (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE ====
==== GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR ====
==== BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF ====
==== LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ====
==== (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT ====
==== OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE ====
==== POSSIBILITY OF SUCH DAMAGE. ====
-------------------------------------------------------------------------*/ 
<'
package pk_rx;

unit uart_rx_monitor_u {
  name : uart_rx_env_name_t;

  scbd_inp : list of byte; --scoreboard
  keep soft scbd_inp.size() == 0;
  scbd_out : list of byte; --scoreboard
  keep soft scbd_out.size() == 0;

  sig : sig_map_u;

  mon_rx_data_cmp() is {
    --The output data may come before the input data.
    var szi : int = scbd_inp.size();
    var szo : int = scbd_out.size();
    check that abs(szi - szo) <= 1 else dut_error(
      "scbd size error i=", szi, " o=", szo
    );
    if(szi != 0 && szo != 0) then {
	  check that scbd_out[0] == scbd_inp[0] else
	  dut_error(
	    "rx_mon_u data compare error dut=", scbd_out[0],
	    " exp=", scbd_inp[0]
	  );
	  messagef(NONE, "%s rx_mon_u date compare ok of %x", name, scbd_out[0]);
	  scbd_inp.delete(0);
	  scbd_out.delete(0);
    };
  };--mon_rx_data_cmp

  fifo_clr_tcm() @sig.clk_r_ev is {
    var del : byte;
    gen del keeping {it >= 0 && it <= 3;};
    wait [del] * cycle;
    sig.full_clr_p$=1;
    wait [1] * cycle;
    sig.full_clr_p$=0;
  };--fifo_clr_tcm

  mon_rx_data_out_tcm() @sig.clk_r_ev is {
    wait true (sig.rst_p$ == 0);
	message(NONE, "mon_rx_out_tcm reset wait ended ", name);
	while(TRUE) {
	  wait true (sig.full_p$ == 1);
	  wait [1] * cycle;
	  messagef(NONE, "%s mon_rx_out_tcm d_out %02x", name, sig.d_out_p$);
      scbd_out.add(sig.d_out_p$);
	  mon_rx_data_cmp();
      start fifo_clr_tcm();
	  wait true (sig.full_p$ == 0);
	};--while
  };--mon_rx_data_out_tcm

  mon_rx_data_inp_tcm() @sig.clk_r_ev is {
    wait true (sig.rst_p$ == 0);
	message(NONE, "mon_rx_inp_tcm reset wait ended ", name);
	while(TRUE) {
	  --wait for start bit
	  var ck_en_cnt : byte = 0;
	  while(ck_en_cnt < 3) {
	    wait true (sig.ck_en_p$ == 1);
		ck_en_cnt=sig.rx_in_p$ == 1 ? ck_en_cnt : ck_en_cnt+1;
	    wait true (sig.ck_en_p$ == 0);
	  };--while

	  --serial data capture 
	  var data_bit : bit;
	  var data_inp : byte = 0;
      var data_bits_l : list of bit;
	  for j from 0 to 7 do {
	    for i from 0 to 2 do {
          wait true (sig.ck_en_p$ == 1);
	      wait true (sig.ck_en_p$ == 0);
		  if(i == 1) then {
		    data_bit=sig.rx_in_p$;
		  };
	    };--for
		data_bits_l.add(data_bit);
	  };--for
      unpack(packing.low, data_bits_l, data_inp);
	  message(NONE, "%s mon_rx_inp_tcm d_inp %02x", name, data_inp);
	  scbd_inp.add(data_inp);
      mon_rx_data_cmp();

	  --wait for stop bit
      wait true (sig.ck_en_p$ == 1);
	  check that sig.rx_in_p$ == 1 else dut_error(
	    name, " mon_rx_inp_tcm stop bit error"
	  );
      wait true (sig.ck_en_p$ == 0);
	};--while
  };--mon_rx_data_inp_tcm

  run() is also {
    start mon_rx_data_out_tcm();
    start mon_rx_data_inp_tcm();
  };


};--unit uart_rx_monitor_u

'>

